@@ -1,7 +1,7 @@
 use crate::config::Config;
 #[cfg(feature = "std")]
 use crate::config::Std;
-use crate::core::Core;
+use crate::core::{Core, Error, What};
 use crate::req::{Req, ReqBuilder};
 use crate::resp::Resp;
 use crate::result_ext::{MapErrInto, ResultExt};
@@ -30,67 +30,26 @@ pub struct Client<Cfg: Config> {
   core: Core<Cfg>,
 }
 
-/// Result of fallible Client operation
-///
-/// `core::result::Result<T, kwap::blocking::client::Error>`
-pub type Result<T> = core::result::Result<T, Error>;
-
 /// Helper methods on Client Results
-pub trait ClientResultExt<T> {
+pub trait ClientResultExt<T, Cfg: Config> {
   /// If we timed out waiting for a response, consider that Ok(None).
   ///
   /// Usually used to handle sending non-confirmable requests that
   /// the server may have received but not responded to.
-  fn timeout_ok(self) -> Result<Option<T>>;
+  fn timeout_ok(self) -> Result<Option<T>, Error<Cfg>>;
 }
 
-impl<T> ClientResultExt<T> for Result<T> {
-  fn timeout_ok(self) -> Result<Option<T>> {
+impl<T, Cfg: Config> ClientResultExt<T, Cfg> for Result<T, Error<Cfg>> {
+  fn timeout_ok(self) -> Result<Option<T>, Error<Cfg>> {
     match self {
       | Ok(t) => Ok(Some(t)),
-      | Err(Error::TimedOut) => Ok(None),
+      | Err(Error { what: What::MessageNeverAcked,
+                    .. }) => Ok(None),
       | Err(e) => Err(e),
     }
   }
 }
 
-/// Errors that could be encountered when sending requests or receiving responses.
-#[derive(Copy, Clone, Debug)]
-pub enum Error {
-  /// There was an issue along the network somewhere
-  NetworkError,
-  /// A message we tried to send was invalid
-  MessageInvalid,
-  /// We timed out waiting for our request to be sent, or for a response to a request.
-  TimedOut,
-  /// The host you provided is not a valid utf8 string
-  HostInvalidUtf8(core::str::Utf8Error),
-  /// The host you provided is not a valid ip address
-  HostInvalidIpAddress,
-  /// Some other error
-  Other,
-}
-
-impl<Cfg: Config> From<crate::core::Error<Cfg>> for Error {
-  fn from(e: crate::core::Error<Cfg>) -> Self {
-    Self::from(&e)
-  }
-}
-
-impl<'a, Cfg: Config> From<&'a crate::core::Error<Cfg>> for Error {
-  fn from(e: &'a crate::core::Error<Cfg>) -> Self {
-    use crate::core::Error::*;
-    match e {
-      | SockError(_) => Self::NetworkError,
-      | ToBytes(_) => Self::MessageInvalid,
-      | MessageNeverAcked => Self::TimedOut,
-      | HostInvalidUtf8(e) => Self::HostInvalidUtf8(*e),
-      | HostInvalidIpAddress => Self::HostInvalidIpAddress,
-      | ClockError => Self::Other,
-    }
-  }
-}
-
 #[cfg(feature = "std")]
 impl Client<Std> {
   /// Create a new Client for a platform supporting Rust's standard library.
@@ -100,16 +59,14 @@ impl Client<Std> {
   /// use kwap::req::ReqBuilder;
   /// use kwap::ContentFormat;
   ///
-  /// fn main() {
-  ///   let mut client = Client::new_std();
-  ///   let req = ReqBuilder::get("127.0.0.1", 5683, "hello").accept(ContentFormat::Text)
-  ///                                                        .build()
-  ///                                                        .unwrap();
+  /// let mut client = Client::new_std();
+  /// let req = ReqBuilder::get("127.0.0.1", 5683, "hello").accept(ContentFormat::Text)
+  ///                                                      .build()
+  ///                                                      .unwrap();
   ///
-  ///   let rep = client.send(req).unwrap();
+  /// let rep = client.send(req).unwrap();
   ///
-  ///   println!("Hello, {}!", rep.payload_string().unwrap());
-  /// }
+  /// println!("Hello, {}!", rep.payload_string().unwrap());
   /// ```
   pub fn new_std() -> Self {
     let clock = crate::std::Clock::new();
@@ -127,7 +84,7 @@ impl<Cfg: Config> Client<Cfg> {
   /// Ping an endpoint
   ///
   /// Note: this will eventually not require Client to be borrowed mutably.
-  pub fn ping(&mut self, host: impl AsRef<str>, port: u16) -> Result<()> {
+  pub fn ping(&mut self, host: impl AsRef<str>, port: u16) -> Result<(), Error<Cfg>> {
     self.core
         .ping(host, port)
         .map_err_into()
@@ -137,11 +94,10 @@ impl<Cfg: Config> Client<Cfg> {
   /// Send a request
   ///
   /// Note: this will eventually not require Client to be borrowed mutably.
-  pub fn send(&mut self, req: Req<Cfg>) -> Result<Resp<Cfg>> {
+  pub fn send(&mut self, req: Req<Cfg>) -> Result<Resp<Cfg>, Error<Cfg>> {
     self.core
         .send_req(req)
-        .map_err_into()
-        .bind(|(token, addr)| nb::block!(self.core.poll_resp(token, addr)).map_err_into())
+        .bind(|(token, addr)| nb::block!(self.core.poll_resp(token, addr)))
   }
 
   /// Send a GET request
